<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaBaseExt\Controller;

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Mail\MailMessage;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * @package     imia_base_ext
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 */
abstract class CommandController extends \TYPO3\CMS\Extbase\Mvc\Controller\CommandController
{
    /**
     * @var array
     */
    protected $hookObjects;

	/**
	 * @var ConfigurationManagerInterface
	 */
	protected $configurationManager;

    /**
     * @var MailMessage
     */
    private $lastSendMessage;

	/**
	 * @param ConfigurationManagerInterface $configurationManager
	 */
	public function injectConfigurationManager(ConfigurationManagerInterface $configurationManager)
    {
		$this->configurationManager = $configurationManager;
		$this->settings = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);
	}

    /**
     * @param integer $targetPageUid
     * @param string $actionName
     * @param array $controllerArguments
     * @param string $controllerName
     * @param string $extensionName
     * @param string $pluginName
     * @param string $format
     * @param string $argumentPrefix
     * @return string
     */
    public function uriFor($targetPageUid, $actionName = null, $controllerArguments = array(), $controllerName = null, $extensionName = null, $pluginName = null, $format = '', $argumentPrefix = null)
    {
        if ($actionName !== null) {
            $controllerArguments['action'] = $actionName;
        }
        if ($controllerName !== null) {
            $controllerArguments['controller'] = $controllerName;
        } else {
            $controllerArguments['controller'] = $this->request->getControllerName();
        }
        if ($extensionName === null) {
            $extensionName = $this->request->getControllerExtensionName();
        }
        if ($pluginName === null && TYPO3_MODE === 'FE') {
            $pluginName = \Tx_Extbase_Utility_Extension::getPluginNameByAction($extensionName, $controllerArguments['controller'], $controllerArguments['action']);
        }
        if ($pluginName === null) {
            $pluginName = $this->request->getPluginName();
        }
        if ($targetPageUid === null && TYPO3_MODE === 'FE') {
            $targetPageUid = \Tx_Extbase_Utility_Extension::getTargetPidByPlugin($extensionName, $pluginName);
        }
        if ($format !== '') {
            $controllerArguments['format'] = $format;
        }
        if ($argumentPrefix !== null) {
            $prefixedControllerArguments = array($argumentPrefix => $controllerArguments);
        } else {
            $pluginNamespace = $this->getPluginNamespace($pluginName, $extensionName);
            $prefixedControllerArguments = array($pluginNamespace => $controllerArguments);
        }

        return $prefixedControllerArguments;
    }

    /**
     * @param integer $targetPageUid
     * @param string $actionName
     * @param array $controllerArguments
     * @param string $controllerName
     * @param string $extensionName
     * @param string $pluginName
     * @param boolean $noCache
     * @param boolean $useCacheHash
     * @param boolean $createAbsoluteUri
     * @param string $format
     * @param string $argumentPrefix
     * @return string
     */
    public function buildFrontendUri($targetPageUid, $actionName = null, $controllerArguments = array(),
        $controllerName = null, $extensionName = null, $pluginName = null, $noCache = false, $useCacheHash = true,
        $createAbsoluteUri = true, $format = '', $argumentPrefix = null)
    {
        $this->initFrontend($targetPageUid);

        $this->uriBuilder->reset();
        $this->uriBuilder
            ->setNoCache($noCache)
            ->setUseCacheHash($useCacheHash)
            ->setCreateAbsoluteUri($createAbsoluteUri)
            ->setArguments($this->uriFor(
                $targetPageUid,
                $actionName,
                $controllerArguments,
                $controllerName,
                $extensionName,
                $pluginName,
                $format,
                $argumentPrefix
            ))
        ;

        return str_replace(PATH_site, '/', $this->uriBuilder->buildFrontendUri());
    }

    /**
     * Translate by key
     *
     * @param string $key
     * @param array $arguments
     * @return string
     */
    public function translate($key, $arguments = array())
    {
        return (string)LocalizationUtility::translate($key, $this->extensionName, $arguments);
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $body
     * @param string $format
     * @param function $closure
     * @return boolean
     */
    public function sendMail(array $recipient, array $sender, $subject, $body, $format = 'plain', $closure = null)
    {
        $message = $this->objectManager->get('TYPO3\CMS\Core\Mail\MailMessage');

        switch ($format) {
            case 'html':
                break;
            case 'plain':
            default:
                $format = 'plain';
                break;
        }

        foreach ($recipient as $key => $value) {
            if (!GeneralUtility::validEmail($key)) {
                if (!GeneralUtility::validEmail($value)) {
                    unset($recipient[$key]);
                }
            }
        }
        foreach ($sender as $key => $value) {
            if (!GeneralUtility::validEmail($key)) {
                if (!GeneralUtility::validEmail($value)) {
                    unset($sender[$key]);
                }
            }
        }
        
        if (count($recipient) > 0 && count($sender) > 0) {
            $message->setTo($recipient)
                ->setFrom($sender)
                ->setSubject($subject)
                ->setBody($body, 'text/' . $format);

            if ($closure) {
                try {
                    $closure($message);
                } catch(\Exception $e) {}
            }

            $message->send();
            $isSend = $message->isSent();

            if ($isSend) {
                $this->setLastSendMessage($message);
            }

            return $isSend;
        } else {
            return false;
        }
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $templateName
     * @param array $arguments
     * @param string $format
     * @param function $closure
     * @return boolean
     */
    public function sendTemplateMailSpecial(array $recipient, array $sender, $subject, $templateName, array $arguments = array(),
                                               $format = 'plain', $closure = null)
    {
        $subject = $this->renderFluidContent($subject, $arguments);

        return $this->sendTemplateMail($recipient, $sender, $subject, $templateName, $arguments, $format, $closure);
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $templateName
     * @param array $arguments
     * @param string $format
     * @param function $closure
     * @return boolean
     */
    public function sendTemplateMail(array $recipient, array $sender, $subject, $templateName, array $arguments = array(),
        $format = 'plain', $closure = null)
    {
        switch ($format) {
            case 'html':
                $ext = 'html';
                break;
            case 'text':
            case 'plain':
            default:
                $ext = 'txt';
                break;
        }

        $message = $this->renderView('Email', $templateName, $arguments, $ext);

        return $this->sendMail(
            $recipient,
            $sender,
            $subject,
            $message,
            $format,
            $closure
        );
    }

    /**
     * @param string $content
     * @param array $arguments
     * @return string
     */
    public function renderFluidContent($content, $arguments = array())
    {
        $tempPath = PATH_site . 'typo3temp/Code/Cache/cache_core/baseext_' . md5($content) . '.temp';
        $tempPathDir = dirname($tempPath);
        if (!is_dir($tempPathDir)) {
            @mkdir($tempPathDir, 0777, true);
        }

        if (!is_dir($tempPathDir)) {
            throw new \Exception('BaseExtRendering, FluidContent: path could not be created: ' . $tempPathDir);
        }

        file_put_contents($tempPath, $content);
        $rendered = $this->renderTemplate($tempPath, $arguments);
        unlink($tempPath);

        return $rendered;
    }

    /**
     * @param string $controller
     * @param string $action
     * @param array $arguments
     * @param string $format
     * @return string
     */
    public function renderView($controller, $action, $arguments, $ext = 'html')
    {
        $templateRootPaths = $this->getPaths('templateRootPath');
        do {
            $templateRootPath = array_shift($templateRootPaths);
            $templatePathAndFilename = $templateRootPath . $controller . '/' . $action . '.' . $ext;
        } while (!file_exists($templatePathAndFilename) && count($templateRootPaths) > 0);

        return $this->renderTemplate($templatePathAndFilename, $arguments);
    }

    /**
     * @param string $controller
     * @param string $action
     * @param array $arguments
     * @param string $format
     * @return string
     */
    public function renderTemplate($templatePathAndFilename, $arguments, $ext = 'html')
    {
        $view = $this->objectManager->get('TYPO3\CMS\Fluid\View\StandaloneView');
        $view->setFormat($ext);
        $view->getRequest()->setControllerExtensionName($this->extensionName);

        $partialRootPaths = $this->getPaths('partialRootPath');
        if (method_exists($view, 'setPartialRootPaths')) {
            $view->setPartialRootPaths($partialRootPaths);
        } else {
            $view->setPartialRootPath($partialRootPaths[0]);
        }
        $layoutRootPaths = $this->getPaths('layoutRootPath');
        if (method_exists($view, 'setLayoutRootPaths')) {
            $view->setLayoutRootPaths($layoutRootPaths);
        } else {
            $view->setLayoutRootPath($layoutRootPaths[0]);
        }

        $view->setTemplatePathAndFilename($templatePathAndFilename);
        if ($arguments['settings'] && is_array($arguments['settings'])) {
            if ($this->settings && is_array($this->settings)) {
                $arguments['settings'] = GeneralUtility::array_merge_recursive_overrule($this->settings, $arguments['settings']);
            }
        } else {
            $arguments['settings'] = $this->settings;
        }

        $view->assignMultiple($arguments);

        return $view->render();
    }

    /**
     * Function: sanitize
     * Returns a sanitized string, typically for URLs.
     *
     * Parameters:
     *     $string - The string to sanitize.
     *     $forceLowercase - Force the string to lowercase?
     *     $anal - If set to *true*, will remove all non-alphanumeric characters.
     */
    public function sanitize($string, $forceLowercase = true, $anal = false)
    {
        $strip = array("~", "`", "!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "_", "=", "+", "[", "{", "]",
            "}", "\\", "|", ";", ":", "\"", "'", "&#8216;", "&#8217;", "&#8220;", "&#8221;", "&#8211;", "&#8212;",
            ",", "<", ".", ">", "/", "?");

        $clean = trim(str_replace($strip, "", strip_tags($string)));
        $clean = preg_replace('/\s+/', "-", $clean);
        $clean = ($anal) ? preg_replace("/[^a-zA-Z0-9]/", "", $clean) : $clean ;

        return ($forceLowercase) ?
            (function_exists('mb_strtolower')) ?
                mb_strtolower($clean, 'UTF-8') :
                strtolower($clean) :
            $clean;
    }

    /**
     * @return \TYPO3\CMS\Core\Mail\MailMessage
     */
    public function getLastSendMessage()
    {
        return $this->lastSendMessage;
    }

    /**
     * @return array
     */
    public function getExtensionConfiguration()
    {
        return $this->configurationManager->getConfiguration(
            ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);
    }

    /**
     * @param type $languageKey
     */
    protected function setLanguage($languageKey)
    {
        if (!$GLOBALS['BE_USER']) {
            $GLOBALS['BE_USER'] = new \stdClass();
            $GLOBALS['BE_USER']->uc = array('lang' => $languageKey);
        }

        $GLOBALS['BE_USER']->uc['lang'] = $languageKey;
    }

    /**
     * Load arguments for a different plugin than the current
     *
     * @param string $pluginName
     */
    protected function loadArguments($pluginName = null)
    {
        if ($pluginName) {
            $arguments = GeneralUtility::_GPmerged($this->getPluginNamespace($pluginName));
        } else {
            $arguments = GeneralUtility::array_merge_recursive_overrule($_GET, $_POST);
        }

        $this->request->setArguments($arguments);
    }

    /**
     *
     * @param string $pluginName
     * @param string $extensionName
     * @return string
     */
    protected function getPluginNamespace($pluginName = null, $extensionName = null)
    {
        $extensionService = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Service\\ExtensionService');
        $pluginNamespace = $extensionService->getPluginNamespace(
            $extensionName ?: $this->extensionName,
            $pluginName ?: $this->request->getPluginName()
        );

        return $pluginNamespace;
    }

    /**
     * @param string $name
     * @return array
     */
    protected function getPaths($name)
    {
        $extensionConfiguration = $this->getExtensionConfiguration();

        $paths = array();
        if (is_array($extensionConfiguration['view']) && array_key_exists('templateRootPaths', $extensionConfiguration['view'])
            && is_array($extensionConfiguration['view'][$name . 's']) && count($extensionConfiguration['view'][$name . 's']) > 0) {
            ksort($extensionConfiguration['view'][$name . 's']);
            foreach ($extensionConfiguration['view'][$name . 's'] as $path) {
                $paths[] = GeneralUtility::getFileAbsFileName($path);
            }
        } else {
            if ($extensionConfiguration['view'][$name]) {
                $paths[] = GeneralUtility::getFileAbsFileName($extensionConfiguration['view'][$name]);
            } else {
                switch ($name) {
                    case 'partialRootPath':
                        $defaultPath = 'Resources/Private/Partials/';
                        break;
                    case 'layoutRootPath':
                        $defaultPath = 'Resources/Private/Layouts/';
                        break;
                    case 'templateRootPath':
                    default:
                        $defaultPath = 'Resources/Private/Templates/';
                        break;
                }

                $paths[] = ExtensionManagementUtility::extPath(
                    GeneralUtility::camelCaseToLowerCaseUnderscored($this->extensionName),
                    $defaultPath
                );
            }
        }

        return $paths;
    }

    /**
     * @param string $method
     * @param array $args
     */
    protected function callHook($method, $args)
    {
        foreach ($this->getHookObjects() as $hookObject) {
            if (method_exists($hookObject, $method)) {
                call_user_func_array(array($hookObject, $method), $args);
            }
        }
    }

    /**
     * @return array
     */
    protected function getHookObjects()
    {
        if (!$this->hookObjects) {
            $this->hookObjects = array();
            $className = get_class($this);
            $className = substr($className, strrpos($className, '\\') + 1);

            $extKey = 'tx_' . strtolower($this->extensionName);
            $controllerName = $this->controllerName ?: str_replace(array('CommandController', 'Controller'), '', $className);
            if (is_array($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$extKey][$controllerName])) {
                foreach ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$extKey][$controllerName] as $_classRef) {
                    $this->hookObjects[] = GeneralUtility::getUserObj($_classRef);
                }
            }
        }

        return $this->hookObjects;
    }

    /**
     * @param MailMessage $message
     * @return $this
     */
    private function setLastSendMessage($message)
    {
        $this->lastSendMessage = $message;

        return $this;
    }

    /**
     * @param integer $pageUid
     */
    protected function initFrontend($pageUid)
    {
        if (!$this->configurationManager->getContentObject()) {
            $this->configurationManager->setContentObject(
                $this->objectManager->get('TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer')
            );
        }

        if (!$GLOBALS['TSFE'] || $GLOBALS['TSFE']->id != $pageUid) {
            $rootline = BackendUtility::BEgetRootLine($pageUid);
            $host = BackendUtility::firstDomainRecord($rootline);

            $_SERVER['HTTP_HOST'] = $host;
            $_SERVER['REQUEST_URI'] = $_SERVER['SCRIPT_NAME'] = 'typo3';

            if (!is_object($GLOBALS['TT'])) {
                $GLOBALS['TT'] = $this->objectManager->get('TYPO3\CMS\Core\TimeTracker\TimeTracker');
                $GLOBALS['TT']->start();
            }

            if (!$GLOBALS['TSFE']) {
                $GLOBALS['TSFE'] = GeneralUtility::makeInstance(
                    'TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController', $GLOBALS['TYPO3_CONF_VARS'], $pageUid, 0, 1);
            }

            $GLOBALS['TSFE']->connectToDB();
            $GLOBALS['TSFE']->initFEuser();
            $GLOBALS['TSFE']->fetch_the_id();
            $GLOBALS['TSFE']->determineId();
            $GLOBALS['TSFE']->getPageAndRootline();
            $GLOBALS['TSFE']->initTemplate();
            $GLOBALS['TSFE']->tmpl->getFileName_backPath = PATH_site;
            $GLOBALS['TSFE']->forceTemplateParsing = 1;
            $GLOBALS['TSFE']->getConfigArray();

            $this->uriBuilder = $this->objectManager->get('TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder');
            $this->uriBuilder->setRequest($this->objectManager->get('TYPO3\CMS\Extbase\Mvc\Web\Request'));
            $this->uriBuilder->initializeObject();
        }
    }
}